@@ -20,7 +20,7 @@
  *                                                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 511845 $                                          *
+ * Version:     $Revision: 562174 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -276,7 +276,7 @@ static int uri_worker_map_realloc(jk_uri_worker_map_t *uw_map)
  * Delete all entries of a given source type
  */
 
-int uri_worker_map_clear(jk_uri_worker_map_t *uw_map,
+static int uri_worker_map_clear(jk_uri_worker_map_t *uw_map,
                          unsigned int source_type, jk_logger_t *l)
 {
     uri_worker_record_t *uwr = NULL;
@@ -535,6 +535,7 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
                               const char *uri, jk_logger_t *l)
 {
     unsigned int i;
+    int reject_unsafe;
     const char *rv = NULL;
     char  url[JK_MAX_URI_LEN+1];
 
@@ -564,6 +565,7 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
     /* Make the copy of the provided uri and strip
      * everything after the first ';' char.
      */
+    reject_unsafe = uw_map->reject_unsafe;
     for (i = 0; i < strlen(uri); i++) {
         if (i == JK_MAX_URI_LEN) {
             jk_log(l, JK_LOG_WARNING,
@@ -574,15 +576,21 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
         }
         if (uri[i] == ';')
             break;
-        else
+        else {
             url[i] = uri[i];
+            if (reject_unsafe && (url[i] == '%' || url[i] == '\\')) {
+                jk_log(l, JK_LOG_INFO, "Potentially unsafe request url '%s' rejected", uri);
+                JK_TRACE_EXIT(l);
+                return NULL;
+            }
+        }
     }
     url[i] = '\0';
 
     if (JK_IS_DEBUG_LEVEL(l)) {
         char *url_rewrite = strstr(uri, JK_PATH_SESSION_IDENTIFIER);
         if (url_rewrite)
-            jk_log(l, JK_LOG_DEBUG, "separating session identifier '%s' from url '%s'",
+            jk_log(l, JK_LOG_DEBUG, "Found session identifier '%s' in url '%s'",
                    url_rewrite, uri);
     }
     if (JK_IS_DEBUG_LEVEL(l))
@@ -656,8 +664,8 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
     jk_map_t *map;
 
     jk_map_alloc(&map);
-    if (jk_map_read_properties(map, uw_map->fname,
-                               &uw_map->modified, 0, l)) {
+    if (jk_map_read_properties(map, uw_map->fname, &uw_map->modified,
+                               JK_MAP_HANDLE_NORMAL, l)) {
         int i;
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
@@ -715,7 +723,7 @@ int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
     if (uw_map->reload > 0 && difftime(now, uw_map->checked) > uw_map->reload) {
         struct stat statbuf;
         uw_map->checked = now;
-        if ((rc = stat(uw_map->fname, &statbuf)) == -1) {
+        if ((rc = jk_stat(uw_map->fname, &statbuf)) == -1) {
             jk_log(l, JK_LOG_ERROR,
                    "Unable to stat the %s (errno=%d)",
                    uw_map->fname, errno);
