@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -17,7 +18,7 @@
 /***************************************************************************
  * Description: URI to worker mapper header file                           *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 405672 $                                           *
+ * Version:     $Revision: 549885 $                                           *
  ***************************************************************************/
 
 #ifndef JK_URI_WORKER_MAP_H
@@ -34,22 +35,37 @@ extern "C"
 #include "jk_logger.h"
 #include "jk_mt.h"
 
-/* Urimap reload time. Use 60 seconds by default.
- */
-#define JK_URIMAP_RELOAD            60
-
 #define MATCH_TYPE_EXACT            0x0001
+/* deprecated
 #define MATCH_TYPE_CONTEXT          0x0002
+ */
 /* match all context path URIs with a path component suffix */
+/* deprecated
 #define MATCH_TYPE_CONTEXT_PATH     0x0004
+ */
+/* deprecated
 #define MATCH_TYPE_SUFFIX           0x0010
+ */
 /* match all URIs of the form *ext */
+/* deprecated
 #define MATCH_TYPE_GENERAL_SUFFIX   0x0020
+ */
 /* match multiple wild characters (*) and (?) */
 #define MATCH_TYPE_WILDCHAR_PATH    0x0040
 #define MATCH_TYPE_NO_MATCH         0x1000
 #define MATCH_TYPE_DISABLED         0x2000
+/* deprecated
 #define MATCH_TYPE_STOPPED          0x4000
+ */
+
+#define SOURCE_TYPE_WORKERDEF       0x0001
+#define SOURCE_TYPE_JKMOUNT         0x0002
+#define SOURCE_TYPE_URIMAP          0x0003
+#define SOURCE_TYPE_DISCOVER        0x0004
+#define SOURCE_TYPE_TEXT_WORKERDEF  ("worker definition")
+#define SOURCE_TYPE_TEXT_JKMOUNT    ("JkMount")
+#define SOURCE_TYPE_TEXT_URIMAP     ("uriworkermap")
+#define SOURCE_TYPE_TEXT_DISCOVER   ("ajp14")
 
 #define JK_MAX_URI_LEN              4095
 struct uri_worker_record
@@ -66,6 +82,9 @@ struct uri_worker_record
     /* Match type */
     unsigned int match_type;
 
+    /* Definition source type */
+    unsigned int source_type;
+
     /* char length of the context */
     size_t context_len;
 };
@@ -92,8 +111,12 @@ struct jk_uri_worker_map
     /* Dynamic config support */
 
     JK_CRIT_SEC cs;
+    /* should we forward potentially unsafe URLs */
+    int reject_unsafe;    
     /* uriworkermap filename */
     const char *fname;    
+    /* uriworkermap reload check interval */
+    int reload;    
     /* Last modified time */
     time_t  modified;
     /* Last checked time */
@@ -101,6 +124,10 @@ struct jk_uri_worker_map
 };
 typedef struct jk_uri_worker_map jk_uri_worker_map_t;
 
+const char *uri_worker_map_get_source(uri_worker_record_t *uwr, jk_logger_t *l);
+
+char *uri_worker_map_get_match(uri_worker_record_t *uwr, char *buf, jk_logger_t *l);
+
 int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map,
                          jk_map_t *init_data, jk_logger_t *l);
 
@@ -110,7 +137,8 @@ int uri_worker_map_open(jk_uri_worker_map_t *uw_map,
                         jk_map_t *init_data, jk_logger_t *l);
 
 int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
-                       const char *puri, const char *pworker, jk_logger_t *l);
+                       const char *puri, const char *worker,
+                       unsigned int source_type, jk_logger_t *l);
 
 const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
                               const char *uri, jk_logger_t *l);
