@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -17,21 +18,26 @@
 /***************************************************************************
  * Description: NSAPI plugin for Netscape servers                          *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 358402 $                                           *
+ * Version:     $Revision: 562175 $                                           *
  ***************************************************************************/
 
 
 #include "nsapi.h"
 #include "jk_global.h"
+#include "jk_url.h"
 #include "jk_util.h"
 #include "jk_map.h"
 #include "jk_pool.h"
 #include "jk_service.h"
 #include "jk_worker.h"
 #include "jk_shm.h"
+#include "jk_ajp13.h"
 
 #define URI_PATTERN "path"
 #define DEFAULT_WORKER_NAME ("ajp13")
+#define REJECT_UNSAFE_TAG   "reject_unsafe"
+
+#define STRNULL_FOR_NULL(x) ((x) ? (x) : "(null)")
 
 struct nsapi_private_data
 {
@@ -89,10 +95,12 @@ static void init_workers_on_other_threads(void *init_d)
     init_map = (jk_map_t *)init_d;
     /* we add the URI->WORKER MAP since workers using AJP14 will feed it */
     /* but where are they here in Netscape ? */
-    if (wc_open(init_map, &worker_env, logger)) {
-        if (uri_worker_map_alloc(&uw_map, NULL, logger)) {
-            uw_map->fname = "";
-            worker_env.uri_to_worker = uw_map;
+    if (uri_worker_map_alloc(&uw_map, NULL, logger)) {
+        uw_map->fname = "";
+        uw_map->reload = JK_URIMAP_DEF_RELOAD;
+        uw_map->reject_unsafe = jk_map_get_bool(init_map, "worker." REJECT_UNSAFE_TAG, JK_FALSE);
+        worker_env.uri_to_worker = uw_map;
+        if (wc_open(init_map, &worker_env, logger)) {
             init_on_other_thread_is_ok = JK_TRUE;
         }
         else {
@@ -223,20 +231,40 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
     char *log_level_str = pblock_findval(JK_LOG_LEVEL_TAG, pb);
     char *log_file = pblock_findval(JK_LOG_FILE_TAG, pb);
     char *shm_file = pblock_findval(JK_SHM_FILE_TAG, pb);
+    char *shm_file_safe = "";
+    char *reject_unsafe = pblock_findval(REJECT_UNSAFE_TAG, pb);
 
     int rc = REQ_ABORTED;
 
-    fprintf(stderr,
-            "In jk_init.\n   Worker file = %s.\n   Log level = %s.\n   Log File = %s\n",
-            worker_prp_file, log_level_str, log_file);
     if (!worker_prp_file) {
         worker_prp_file = JK_WORKER_FILE_DEF;
     }
 
     if (!log_level_str) {
-        log_level_str = JK_LOG_LEVEL_DEF;
+        log_level_str = JK_LOG_DEF_VERB;
     }
 
+    if (!log_file) {
+        fprintf(stderr,
+                "Missing attribute %s in magnus.conf (jk_init) - aborting!\n", JK_LOG_FILE_TAG);
+        return rc;
+    }
+
+    if (shm_file) {
+        shm_file_safe = shm_file;
+    }
+#if !defined(WIN32) && !defined(NETWARE)
+    else {
+        fprintf(stderr,
+                "Missing attribute %s in magnus.conf (jk_init) - aborting!\n", JK_SHM_FILE_TAG);
+        return rc;
+    }
+#endif
+
+    fprintf(stderr,
+            "In jk_init.\n   Worker file = %s.\n   Log level = %s.\n   Log File = %s\n   SHM File = %s\n",
+            worker_prp_file, log_level_str, log_file, shm_file);
+
     if (!jk_open_file_logger(&logger, log_file,
                              jk_parse_log_level(log_level_str))) {
         logger = NULL;
@@ -244,10 +272,18 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
     
     jk_shm_open(shm_file, JK_SHM_DEF_SIZE, logger);
     if (jk_map_alloc(&init_map)) {
-        if (jk_map_read_properties(init_map, worker_prp_file, NULL)) {
+        if (jk_map_read_properties(init_map, worker_prp_file, NULL,
+                                   JK_MAP_HANDLE_DUPLICATES, logger)) {
             int sleep_cnt;
             SYS_THREAD s;
 
+            if (jk_map_resolve_references(init_map, "worker.", 1, 1, logger) == JK_FALSE) {
+                jk_log(logger, JK_LOG_ERROR, "Error in resolving configuration references");
+            }
+
+            if (reject_unsafe) {
+                jk_map_add(init_map, "worker." REJECT_UNSAFE_TAG, reject_unsafe); 
+            }
             s = systhread_start(SYSTHREAD_DEFAULT_PRIORITY,
                                 0, init_workers_on_other_threads, init_map);
             for (sleep_cnt = 0; sleep_cnt < 60; sleep_cnt++) {
@@ -261,6 +297,8 @@ NSAPI_PUBLIC int jk_init(pblock * pb, Session * sn, Request * rq)
             if (init_on_other_thread_is_done && init_on_other_thread_is_ok) {
                 magnus_atrestart(jk_term, NULL);
                 rc = REQ_PROCEED;
+                jk_log(logger, JK_LOG_INFO, "nsapi_redirector/%s initialized",
+                       JK_VERSTRING);
             }
 
 /*            if(wc_open(init_map, NULL, logger)) {
@@ -290,14 +328,15 @@ NSAPI_PUBLIC void jk_term(void *p)
         uri_worker_map_free(&uw_map, logger);
     }
 
+    if (init_map) {
+        jk_map_free(&init_map);
+    }
+
+    jk_shm_close();
     wc_close(logger);
     if (logger) {
         jk_close_file_logger(&logger);
     }
-
-	if (init_map) {
-        jk_map_free(&init_map);
-    }
 }
 
 NSAPI_PUBLIC int jk_service(pblock * pb, Session * sn, Request * rq)
@@ -336,15 +375,34 @@ NSAPI_PUBLIC int jk_service(pblock * pb, Session * sn, Request * rq)
 
         s.ws_private = &private_data;
         s.pool = &private_data.p;
+        s.retries = worker->retries;
 
         wc_maintain(logger);
         if (init_ws_service(&private_data, &s)) {
             jk_endpoint_t *e = NULL;
             if (worker->get_endpoint(worker, &e, logger)) {
-                int recover = JK_FALSE;
-                if (e->service(e, &s, logger, &recover)) {
+                int is_error = JK_HTTP_SERVER_ERROR;
+                int result;
+                if ((result = e->service(e, &s, logger, &is_error)) > 0) {
                     rc = REQ_PROCEED;
+                    if (JK_IS_DEBUG_LEVEL(logger))
+                        jk_log(logger, JK_LOG_DEBUG,
+                               "service() returned OK");
                 }
+                else {
+                    protocol_status(sn, rq, is_error, NULL);
+                    if ((result == JK_CLIENT_ERROR) && (is_error == JK_HTTP_OK)) {
+                        rc = REQ_EXIT;
+                        jk_log(logger, JK_LOG_INFO,
+                               "service() failed because client aborted connection");
+                    }
+                    else {
+                        rc = REQ_ABORTED;
+                        jk_log(logger, JK_LOG_ERROR,
+                               "service() failed with http error %d", is_error);
+                    }
+                }
+
                 e->done(&e, logger);
             }
         }
@@ -358,13 +416,17 @@ static int init_ws_service(nsapi_private_data_t * private_data,
                            jk_ws_service_t *s)
 {
     char *tmp;
+    int size;
     int rc;
 
-    s->jvm_route = NULL;
+    s->route = NULL;
     s->start_response = start_response;
     s->read = ws_read;
     s->write = ws_write;
     s->flush = NULL;
+    s->flush_packets = JK_FALSE;
+    s->flush_header = JK_FALSE;
+    s->disable_reuse = JK_FALSE;
 
     /* Clear RECO status */
     s->reco_status = RECO_NONE;
@@ -378,7 +440,7 @@ static int init_ws_service(nsapi_private_data_t * private_data,
                         &tmp, private_data->sn, private_data->rq);
 
     if ((rc != REQ_ABORTED) && tmp) {
-        s->content_length = atoi(tmp);
+        sscanf(tmp, "%" JK_UINT64_T_FMT, &(s->content_length));
     }
 
     s->method = pblock_findval("method", private_data->rq->reqpb);
@@ -387,7 +449,11 @@ static int init_ws_service(nsapi_private_data_t * private_data,
     s->remote_host = session_dns(private_data->sn);
     s->remote_addr = pblock_findval("ip", private_data->sn->client);
 
-    s->req_uri = pblock_findval("uri", private_data->rq->reqpb);
+    tmp = pblock_findval("uri", private_data->rq->reqpb);
+    size = 3 * strlen(tmp) + 1;
+    s->req_uri = jk_pool_alloc(s->pool, size);
+    jk_canonenc(tmp, s->req_uri, size);
+
     s->query_string = pblock_findval("query", private_data->rq->reqpb);
 
     s->server_name = server_hostname;
@@ -408,7 +474,7 @@ static int init_ws_service(nsapi_private_data_t * private_data,
     s->headers_names = NULL;
     s->headers_values = NULL;
     s->num_headers = 0;
-	s->uw_map = uw_map;
+    s->uw_map = uw_map;
 
 #ifdef NETWARE
     /* on NetWare, we can have virtual servers that are secure.
@@ -443,7 +509,27 @@ static int init_ws_service(nsapi_private_data_t * private_data,
         s->ssl_session = NULL;
     }
 
-    return setup_http_headers(private_data, s);
+    rc = setup_http_headers(private_data, s);
+
+    /* Dump all connection param so we can trace what's going to
+     * the remote tomcat
+     */
+    if (JK_IS_DEBUG_LEVEL(logger)) {
+        jk_log(logger, JK_LOG_DEBUG,
+               "Service protocol=%s method=%s host=%s addr=%s name=%s port=%d auth=%s user=%s uri=%s",
+               STRNULL_FOR_NULL(s->protocol),
+               STRNULL_FOR_NULL(s->method),
+               STRNULL_FOR_NULL(s->remote_host),
+               STRNULL_FOR_NULL(s->remote_addr),
+               STRNULL_FOR_NULL(s->server_name),
+               s->server_port,
+               STRNULL_FOR_NULL(s->auth_type),
+               STRNULL_FOR_NULL(s->remote_user),
+               STRNULL_FOR_NULL(s->req_uri));
+    }
+
+    return rc;
+
 }
 
 static int setup_http_headers(nsapi_private_data_t * private_data,
